window.iam = {

    submitWithButton: function (button) {
        button = $(button);
        var name = button.attr('name');
        var form = button.closest('form');
        this.submitWithAction(form, name);
    },

    submitWithAction: function (form, action) {
        this.submitForm(form, {
            action: action
        });
    },

    submitForm: function (form, options) {
        options = options || {};
        if (options.action) {
            this.appendHiddenField(form, options.action, true);
        }
        form.submit();
    },

    appendHiddenField: function (form, name, value) {
        $('<input>').attr({
            type: 'hidden',
            name: name,
            value: value
        }).appendTo(form);
    },

    /*
     * Delays the appearance of all elements with an "data-display-delay" attribute.
     */
    displayDelay: function () {
        var delayElements = $("[data-display-delay]");
        for (var i = 0; i < delayElements.length; i++) {
            var element = $(delayElements[i]);
            var delay = parseInt(element.attr("data-display-delay"));
            // delay == 0: don't do anything which implies that the button is shown immediately
            if (delay > 0) {
                // Positive delay: let the button appear after a while
                element.hide();
                setTimeout(function () {
                    element.show();
                }, delay);
            } else if (delay < 0) {
                // Negative delay: remove the button since it's never shown
                element.remove();
            }
        }
    },

    /**
     * Puts the focus on the most preferable element.
     */
    autofocus: function () {
        var focusable = ':visible:not([disabled]):not([readonly])';
        var errorFields = $('.has-danger input, .has-danger select').filter(focusable);
        var inputFields = $('input,select').filter(focusable);
        var buttons = $('.button').filter(focusable);

        if (errorFields.length) {
            errorFields.first().focus();
        } else if (inputFields.length) {
            inputFields.first().focus();
        } else if (buttons.length === 1) {
            buttons.first().focus();
        }
    },

    /**
     * When a form is submitted, prevent all buttons within the form to be clicked by disabling them.
     */
    doubleClickPrevention: {
        enabled: true,

        apply: function () {
            if (!iam.doubleClickPrevention.enabled) {
                return;
            }

            $('form').on('submit', function () {
                let form = this;

                if ($(form).prop('data-disabled')) {
                    return false;
                }

                $(form).prop('data-disabled', true);
                $(form).find('.button').addClass('disabled');
            });
        }
    }
};

$(function () {
    $('.submit-with-button').on('click', function () {
        iam.submitWithButton(this)
    });
    iam.autofocus();
    iam.displayDelay();
    iam.doubleClickPrevention.apply();
});
